@@ -58,6 +58,7 @@ class PreparationReport(object):
         self.checkout = True     # checkout vs update
         self.backup = False      # backup vs delete
         self.backup_path = None  # where to move tree to
+        self.inplace = False     # whether to follow symlink or just delete
 
 
 ## Each Config element provides actions on a local folder
@@ -100,7 +101,7 @@ class ConfigElement(object):
         preparation_report.skip = True
         return preparation_report
 
-    def install(self, checkout=True, backup=False, backup_path=None, verbose=False):
+    def install(self, checkout=True, backup=False, backup_path=None, inplace=False, verbose=False):
         """
         Attempt to make it so that self.path is the result of checking
         out / updating from remote repo.
@@ -123,7 +124,8 @@ class ConfigElement(object):
 
     def get_versioned_path_spec(self):
         """PathSpec where VCS elements have the version looked up"""
-        raise NotImplementedError("ConfigElement get_versioned_path_spec unimplemented")
+        raise NotImplementedError(
+            "ConfigElement get_versioned_path_spec unimplemented")
 
     def is_vcs_element(self):
         # subclasses to override when appropriate
@@ -139,14 +141,14 @@ class ConfigElement(object):
         if not backup_path:
             raise MultiProjectException(
                 "[%s] Cannot install %s.  backup disabled." % (self.get_local_name(),
-                                                             self.get_path()))
+                                                               self.get_path()))
         backup_path = os.path.join(
             backup_path,
             "%s_%s" % (os.path.basename(self.path),
-                     datetime.datetime.now().strftime("%Y-%m-%d-%H-%M-%S")))
+                       datetime.datetime.now().strftime("%Y-%m-%d-%H-%M-%S")))
         print("[%s] Backing up %s to %s" % (self.get_local_name(),
-                                          self.get_path(),
-                                          backup_path))
+                                            self.get_path(),
+                                            backup_path))
         shutil.move(self.path, backup_path)
 
     def __str__(self):
@@ -166,7 +168,7 @@ class OtherConfigElement(ConfigElement):
         self.uri = uri
         self.version = version
 
-    def install(self, checkout=True, backup=False, backup_path=None, verbose=False):
+    def install(self, checkout=True, backup=False, backup_path=None, inplace=False, verbose=False):
         return True
 
     def get_versioned_path_spec(self):
@@ -198,7 +200,7 @@ class SetupConfigElement(ConfigElement):
     configuration data for a config.
     """
 
-    def install(self, checkout=True, backup=False, backup_path=None, verbose=False):
+    def install(self, checkout=True, backup=False, backup_path=None, inplace=False, verbose=False):
         return True
 
     def get_versioned_path_spec(self):
@@ -261,12 +263,14 @@ class VCSConfigElement(ConfigElement):
         preparation_report = PreparationReport(self)
         present = self.detect_presence()
         if present or self.path_exists():
+            is_link = os.path.islink(self.path)
             # Directory exists see what we need to do
             error_message = None
-
             if not present:
                 error_message = "Failed to detect %s presence at %s." % (
                     self.get_vcs_type_name(), self.path)
+                if is_link:
+                    error_message += " Path is symlink, only symlink will be removed."
             else:
                 cur_url = self._get_vcsc().get_url()
                 if cur_url is not None:
@@ -287,35 +291,48 @@ class VCSConfigElement(ConfigElement):
                 # If robust ala continue-on-error, just error now and
                 # it will be continued at a higher level
                 if robust:
-                    raise MultiProjectException("Update Failed of %s: %s" % (
-                            self.path, error_message))
+                    raise MultiProjectException("Update Failed of %s: %s" %
+                                                (self.path, error_message))
                 # prompt the user based on the error code
                 if arg_mode == 'prompt':
-                    print("Prepare updating %s (version %s) to %s" % (
-                            self.uri, self.version, self.path))
-                    mode = Ui.get_ui().prompt_del_abort_retry(error_message,
-                                                              allow_skip=True)
+                    print("Prepare updating %s (version %s) to %s" %
+                          (self.uri, self.version, self.path))
+                    mode = Ui.get_ui().prompt_del_abort_retry(
+                        error_message,
+                        allow_skip=True,
+                        allow_inplace=is_link)
                 else:
                     mode = arg_mode
                 if mode == 'backup':
                     preparation_report.backup = True
                     if backup_path is None:
-                        print("Prepare updating %s (version %s) to %s" % (
-                                self.uri, self.version, self.path))
-                        preparation_report.backup_path = Ui.get_ui().get_backup_path()
+                        print("Prepare updating %s (version %s) to %s" %
+                              (self.uri, self.version, self.path))
+                        preparation_report.backup_path = \
+                            Ui.get_ui().get_backup_path()
                     else:
                         preparation_report.backup_path = backup_path
-                if mode == 'abort':
+                elif mode == 'abort':
                     preparation_report.abort = True
                     preparation_report.error = error_message
-                if mode == 'skip':
+                elif mode == 'skip':
                     preparation_report.skip = True
                     preparation_report.error = error_message
-                if mode == 'delete':
+                elif mode == 'delete':
                     preparation_report.backup = False
+                elif mode == 'inplace':
+                    preparation_report.inplace = True
+                else:
+                    raise RuntimeError(
+                        'Bug: Unknown option "%s" selected' % mode)
         return preparation_report
 
-    def install(self, checkout=True, backup=True, backup_path=None, verbose=False):
+    def install(self,
+                checkout=True,
+                backup=True,
+                backup_path=None,
+                inplace=False,
+                verbose=False):
         """
         Runs the equivalent of SCM checkout for new local repos or
         update for existing.
@@ -326,15 +343,24 @@ class VCSConfigElement(ConfigElement):
         if backup is false folder will be DELETED.
         :param backup_path: if checkout is true and backup is true,
         move folder to this location
+        :param inplace: for symlinks, allows to delete contents
+        at target location and checkout to there.
         """
         if checkout is True:
-            print("[%s] Installing %s (version %s) to %s" % (
-                    self.get_local_name(), self.uri, self.version, self.get_path()))
+            print("[%s] Fetching %s (version %s) to %s" % (
+                self.get_local_name(), self.uri, self.version, self.get_path()))
             if self.path_exists():
-                if (backup is False):
-                    shutil.rmtree(self.path)
+                if os.path.islink(self.path):
+                    if inplace is False:
+                        # remove same as unlink
+                        os.remove(self.path)
+                    else:
+                        shutil.rmtree(os.path.realpath(self.path))
                 else:
-                    self.backup(backup_path)
+                    if backup is False:
+                        shutil.rmtree(self.path)
+                    else:
+                        self.backup(backup_path)
             if not self._get_vcsc().checkout(self.uri,
                                              self.version,
                                              verbose=verbose):
@@ -345,11 +371,12 @@ class VCSConfigElement(ConfigElement):
                         self.version,
                         self.get_path()))
         else:
-            print("[%s] Updating %s" % (self.get_local_name(), self.get_path()))
+            print("[%s] Updating %s" %
+                  (self.get_local_name(), self.get_path()))
             if not self._get_vcsc().update(self.version, verbose=verbose):
                 raise MultiProjectException(
                     "[%s] Update Failed of %s" % (self.get_local_name(),
-                                                self.get_path()))
+                                                  self.get_path()))
         print("[%s] Done." % self.get_local_name())
 
     def get_path_spec(self):
@@ -430,7 +457,7 @@ class AVCSConfigElement(VCSConfigElement):
         # to make more use of lazy initializer, do not instantiate
         # client for just this this is crucial for bash tab completion
         if self.get_vcs_type_name() == 'git':
-            return os.path.isdir(os.path.join(self.path, '.git'))
+            return os.path.exists(os.path.join(self.path, '.git'))
         elif self.get_vcs_type_name() == 'svn':
             return os.path.isdir(os.path.join(self.path, '.svn'))
         elif self.get_vcs_type_name() == 'hg':
