@@ -43,12 +43,13 @@ A change to expect is abstraction of user interaction.
 """
 
 
+import sys
 import os
 from rosinstall.common import MultiProjectException, DistributedWork, \
     select_elements, normabspath
 from rosinstall.config import Config, realpath_relation
 from rosinstall.config_yaml import aggregate_from_uris, generate_config_yaml, \
-    get_path_specs_from_uri
+    get_path_specs_from_uri, PathSpec
 
 import vcstools
 import vcstools.__version__
@@ -80,7 +81,7 @@ def get_config(basepath,
     if basepath is None:
         raise MultiProjectException("Need to provide a basepath for Config.")
 
-    #print("source...........................", path_specs)
+    # print("source...........................", path_specs)
 
     ## Generate the config class with the uri and path
     if (config_filename is not None
@@ -102,33 +103,46 @@ def get_config(basepath,
     return config
 
 
-def add_uris(config, additional_uris, merge_strategy="KillAppend"):
+def add_uris(config,
+             additional_uris,
+             merge_strategy="KillAppend",
+             allow_other_element=True):
     """
     changes the given config by merging with the additional_uris
 
     :param config: a Config objects
     :param additional_uris: the location of config specifications or folders
-    :param config_filename: name of files which may be looked at for config information
+    :param config_filename: name of files which may be looked at for config
+    information
     :param merge_strategy: One of 'KillAppend, 'MergeKeep', 'MergeReplace'
-    :returns: a dict {<local-name>: (<action>, <path-spec>), <local-name>: ...} determined by the merge_strategy
+    :param allow_other_element: if False, discards elements to be added with
+    no SCM information
+    :returns: a dict {<local-name>: (<action>, <path-spec>), <local-name>: ...}
+    determined by the merge_strategy
     :raises MultiProjectException: on plenty of errors
     """
     if config is None:
         raise MultiProjectException("Need to provide a Config.")
 
-    if additional_uris is None or len(additional_uris) == 0:
+    if not additional_uris:
         return {}
 
-    added_uris = []
-    if config.get_config_filename() is not None:
+    if config.get_config_filename() is None:
+        added_uris = additional_uris
+    else:
+        added_uris = []
+        # reject if the additional uri points to the same file as our
+        # config is based on
         for uri in additional_uris:
+            # check whether we try to merge with other workspace
             comp_uri = None
             if (os.path.isfile(uri)
                 and os.path.basename(uri) == config.get_config_filename()):
-
+                # add from other workspace by file
                 comp_uri = os.path.dirname(uri)
             if (os.path.isdir(uri)
                 and os.path.isfile(os.path.join(uri, config.get_config_filename()))):
+                # add from other workspace by dir
                 comp_uri = uri
             if (comp_uri is not None and
                 realpath_relation(os.path.abspath(comp_uri),
@@ -137,12 +151,14 @@ def add_uris(config, additional_uris, merge_strategy="KillAppend"):
                 continue
             added_uris.append(uri)
 
-    path_specs = aggregate_from_uris(added_uris, config.get_config_filename())
-
     actions = {}
-    for path_spec in path_specs:
-        action = config.add_path_spec(path_spec, merge_strategy)
-        actions[path_spec.get_local_name()] = (action, path_spec)
+    if len(added_uris) > 0:
+        path_specs = aggregate_from_uris(added_uris,
+                                         config.get_config_filename(),
+                                         allow_other_element)
+        for path_spec in path_specs:
+            action = config.add_path_spec(path_spec, merge_strategy)
+            actions[path_spec.get_local_name()] = (action, path_spec)
 
     return actions
 
@@ -164,11 +180,11 @@ Git:       %s
 Tar:       %s
 Bzr:       %s
 """ % (vcstools.__version__.version,
-     prettyversion(vcstools.SvnClient.get_environment_metadata()),
-     prettyversion(vcstools.HgClient.get_environment_metadata()),
-     prettyversion(vcstools.GitClient.get_environment_metadata()),
-     prettyversion(vcstools.TarClient.get_environment_metadata()),
-     prettyversion(vcstools.BzrClient.get_environment_metadata()))
+       prettyversion(vcstools.SvnClient.get_environment_metadata()),
+       prettyversion(vcstools.HgClient.get_environment_metadata()),
+       prettyversion(vcstools.GitClient.get_environment_metadata()),
+       prettyversion(vcstools.TarClient.get_environment_metadata()),
+       prettyversion(vcstools.BzrClient.get_environment_metadata()))
 
 
 def cmd_status(config, localnames=None, untracked=False):
@@ -202,8 +218,8 @@ def cmd_status(config, localnames=None, untracked=False):
                 status_aligned = ''
                 for line in status.splitlines():
                     status_aligned = "%s%s%s\n" % (status_aligned,
-                                                 line[:columns].ljust(8),
-                                                 line[columns:])
+                                                   line[:columns].ljust(8),
+                                                   line[columns:])
                 status = status_aligned
             return {'status': status}
 
@@ -277,20 +293,24 @@ def cmd_install_or_update(
         if backup_path is not None:
             abs_backup_path = os.path.join(config.get_base_path(), backup_path)
         try:
-            preparation_report = tree_el.prepare_install(backup_path=abs_backup_path,
-                                                   arg_mode=mode,
-                                                   robust=robust)
+            preparation_report = tree_el.prepare_install(
+                backup_path=abs_backup_path,
+                arg_mode=mode,
+                robust=robust)
             if preparation_report is not None:
                 if preparation_report.abort:
-                    raise MultiProjectException("Aborting install because of %s" % preparation_report.error)
+                    raise MultiProjectException(
+                        "Aborting install because of %s" % preparation_report.error)
                 if not preparation_report.skip:
                     preparation_reports.append(preparation_report)
                 else:
                     if preparation_report.error is not None:
-                        print("Skipping install of %s because: %s" % (preparation_report.config_element.get_local_name(),
-                                                                    preparation_report.error))
+                        print("Skipping install of %s because: %s" %
+                              (preparation_report.config_element.get_local_name(),
+                               preparation_report.error))
         except MultiProjectException as exc:
-            fail_str = "Failed to install tree '%s'\n %s" % (tree_el.get_path(), exc)
+            fail_str = ("Failed to install tree '%s'\n %s" %
+                        (tree_el.get_path(), exc))
             if robust:
                 success = False
                 print("Continuing despite %s" % fail_str)
@@ -307,6 +327,7 @@ def cmd_install_or_update(
             self.element.install(checkout=self.report.checkout,
                                  backup=self.report.backup,
                                  backup_path=self.report.backup_path,
+                                 inplace=self.report.inplace,
                                  verbose=self.report.verbose)
             return {}
 
@@ -322,22 +343,39 @@ def cmd_install_or_update(
         print ("Exception caught during install: %s" % exc)
         success = False
         if not robust:
-            raise exc
+            raise
     return success
-    # TODO go back and make sure that everything in options.path is described
-    # in the yaml, and offer to delete otherwise? not sure, but it could go here
+    # TODO go back and make sure that everything in options.path is
+    # described in the yaml, and offer to delete otherwise? not sure,
+    # but it could go here
 
 
 def cmd_snapshot(config, localnames=None):
     elements = select_elements(config, localnames)
     source_aggregate = []
     for element in elements:
-        source = element.get_versioned_path_spec().get_legacy_yaml()
-        source_aggregate.append(source)
+        if element.is_vcs_element():
+            spec = element.get_versioned_path_spec()
+            export_spec = PathSpec(
+                local_name=spec.get_local_name(),
+                scmtype=spec.get_scmtype(),
+                uri=spec.get_uri() or spec.get_curr_uri(),
+                version=(spec.get_current_revision() or
+                         spec.get_revision() or
+                         spec.get_version()),
+                path=spec.get_path())
+            if not export_spec.get_version():
+                sys.stderr.write(
+                    'Warning, discarding non-vcs element %s\n' % element.get_local_name())
+            source = export_spec.get_legacy_yaml()
+            source_aggregate.append(source)
+        else:
+            sys.stderr.write('Warning, discarding non-vcs element %s\n' %
+                             element.get_local_name())
     return source_aggregate
 
 
-def cmd_info(config, localnames=None):
+def cmd_info(config, localnames=None, untracked=False):
     """This function compares what should be (config_file) with what is
     (directories) and returns a list of dictionary giving each local
     path and all the state information about it available.
@@ -345,9 +383,10 @@ def cmd_info(config, localnames=None):
 
     class InfoRetriever():
 
-        def __init__(self, element, path):
+        def __init__(self, element, path, untracked):
             self.element = element
             self.path = path
+            self.untracked = untracked
 
         def do_work(self):
             localname = ""
@@ -363,9 +402,9 @@ def cmd_info(config, localnames=None):
             path = self.element.get_path() or localname
 
             if localname is None or localname == "":
-                raise MultiProjectException("Missing local-name in element: %s" % self.element)
-            abs_path = normabspath(path, self.path)
-            if (os.path.exists(abs_path)):
+                raise MultiProjectException(
+                    "Missing local-name in element: %s" % self.element)
+            if (os.path.exists(normabspath(path, self.path))):
                 exists = True
             if self.element.is_vcs_element():
                 if not exists:
@@ -375,7 +414,7 @@ def cmd_info(config, localnames=None):
                     path_spec = self.element.get_versioned_path_spec()
                     version = path_spec.get_version()
                     curr_uri = path_spec.get_curr_uri()
-                    status = self.element.get_status(self.path)
+                    status = self.element.get_status(self.path, self.untracked)
                     if (status is not None and
                         status.strip() != ''):
                         modified = True
@@ -406,6 +445,6 @@ def cmd_info(config, localnames=None):
     elements = select_elements(config, localnames)
     for element in elements:
         if element.get_properties() is None or not 'setup-file' in element.get_properties():
-            work.add_thread(InfoRetriever(element, path))
+            work.add_thread(InfoRetriever(element, path, untracked))
     outputs = work.run()
     return outputs
