@@ -47,7 +47,8 @@ import yaml
 
 from optparse import OptionParser
 
-from rosinstall.cli_common import get_info_list, get_info_table, get_workspace
+from rosinstall.cli_common import get_info_list, get_info_table, \
+    get_info_table_raw_csv, get_workspace, ONLY_OPTION_VALID_ATTRS
 import rosinstall.rosinstall_cmd as rosinstall_cmd
 from rosinstall.multiproject_cmd import get_config, cmd_install_or_update, \
     cmd_snapshot, cmd_version, cmd_info
@@ -81,15 +82,17 @@ class RoswsCLI(MultiprojectCLI):
         MultiprojectCLI.__init__(self,
                                  progname=progname,
                                  config_filename=config_filename,
+                                 allow_other_element=True,
                                  config_generator=rosinstall_cmd.cmd_persist_config)
 
     def cmd_init(self, argv):
         if self.config_filename is None:
             print('Error: Bug: config filename required for init')
             return 1
-        parser = OptionParser(usage="""usage: %s init [TARGET_PATH [SOURCE_PATH]]?""" % _PROGNAME,
-                              formatter=IndentedHelpFormatterWithNL(),
-                              description=__MULTIPRO_CMD_DICT__["init"] + """
+        parser = OptionParser(
+            usage="""usage: %s init [TARGET_PATH [SOURCE_PATH]]?""" % self.progname,
+            formatter=IndentedHelpFormatterWithNL(),
+            description=__MULTIPRO_CMD_DICT__["init"] + """
 
 %(prog)s init does the following:
   1. Reads folder/file/web-uri SOURCE_PATH looking for a rosinstall yaml
@@ -101,8 +104,8 @@ If PATH is not given, uses current dir.
 
 Examples:
 $ %(prog)s init ~/fuerte /opt/ros/fuerte
-""" % {'cfg_file': self.config_filename, 'prog': _PROGNAME},
-                              epilog="See: http://www.ros.org/wiki/rosinstall for details\n")
+""" % {'cfg_file': self.config_filename, 'prog': self.progname},
+            epilog="See: http://www.ros.org/wiki/rosinstall for details\n")
         parser.add_option("-c", "--catkin", dest="catkin", default=False,
                           help="Declare this is a catkin build.",
                           action="store_true")
@@ -128,7 +131,8 @@ $ %(prog)s init ~/fuerte /opt/ros/fuerte
                 print('Error: Cannot create in target path %s ' % target_path)
 
         if os.path.exists(os.path.join(target_path, self.config_filename)):
-            print('Error: There already is a workspace config file %s at "%s". Use %s install/modify.' % (self.config_filename, target_path, _PROGNAME))
+            print('Error: There already is a workspace config file %s at "%s". Use %s install/modify.' %
+                  (self.config_filename, target_path, self.progname))
             return 1
         if len(args) > 2:
             parser.error('Too many arguments')
@@ -144,7 +148,8 @@ $ %(prog)s init ~/fuerte /opt/ros/fuerte
 
         # includes ROS specific files
 
-        print("Writing %s" % os.path.join(config.get_base_path(), self.config_filename))
+        print("Writing %s" % os.path.join(config.get_base_path(),
+              self.config_filename))
         rosinstall_cmd.cmd_persist_config(config)
 
         ## install or update each element
@@ -169,9 +174,8 @@ $ %(prog)s init ~/fuerte /opt/ros/fuerte
             print("\nType 'source %s/setup.bash' to change into this environment. Add that source command to the bottom of your ~/.bashrc to set it up every time you log in.\n\nIf you are not using bash please see http://www.ros.org/wiki/rosinstall/NonBashShells " % os.path.abspath(target_path))
         return 0
 
-
     def cmd_regenerate(self, target_path, argv, config=None):
-        parser = OptionParser(usage="usage: %s regenerate" % _PROGNAME,
+        parser = OptionParser(usage="usage: %s regenerate" % self.progname,
                               formatter=IndentedHelpFormatterWithNL(),
                               description=__MULTIPRO_CMD_DICT__["remove"] + """
 
@@ -189,6 +193,10 @@ accidentally.
         parser.add_option("--cmake-prefix-path", dest="catkinpp", default=None,
                           help="Where to set the CMAKE_PREFIX_PATH",
                           action="store")
+        # -t option required here for help but used one layer above, see cli_common
+        parser.add_option("-t", "--target-workspace", dest="workspace", default=None,
+                          help="which workspace to use",
+                          action="store")
         (options, args) = parser.parse_args(argv)
         if len(args) > 0:
             print("Error: Too many arguments.")
@@ -203,7 +211,7 @@ accidentally.
         elif config.get_base_path() != target_path:
             raise MultiProjectException(
                 "Config path does not match %s %s " % (config.get_base_path(),
-                                                     target_path))
+                                                       target_path))
         rosinstall_cmd.cmd_generate_ros_files(config,
                                               target_path,
                                               nobuild=True,
@@ -214,10 +222,12 @@ accidentally.
         return 0
 
     def cmd_info(self, target_path, argv, reverse=True, config=None):
-        only_option_valid_attrs = ['path', 'localname', 'version', 'revision', 'cur_revision', 'uri', 'cur_uri', 'scmtype']
-        parser = OptionParser(usage="usage: %s info [localname]* [OPTIONS]" % _PROGNAME,
-                              formatter=IndentedHelpFormatterWithNL(),
-                              description=__MULTIPRO_CMD_DICT__["info"] + """
+        # similar to multiproject_cli except shows ros-pkg-path
+        # options
+        parser = OptionParser(
+            usage="usage: %s info [localname]* [OPTIONS]" % self.progname,
+            formatter=IndentedHelpFormatterWithNL(),
+            description=__MULTIPRO_CMD_DICT__["info"] + """
 
 The Status (S) column shows
  x  for missing
@@ -245,28 +255,38 @@ $ %(prog)s info -t ~/ros/fuerte
 $ %(prog)s info robot_model
 $ %(prog)s info --yaml
 $ %(prog)s info --only=path,cur_uri,cur_revision robot_model geometry
-""" % {'prog': _PROGNAME, 'opts': only_option_valid_attrs},
-                              epilog="See: http://www.ros.org/wiki/rosinstall for details\n")
-        parser.add_option("--data-only", dest="data_only", default=False,
-                          help="Does not provide explanations",
-                          action="store_true")
-        parser.add_option("--no-pkg-path", dest="no_pkg_path", default=False,
-                          help="Suppress ROS_PACKAGE_PATH.",
-                          action="store_true")
-        parser.add_option("--pkg-path-only", dest="pkg_path_only", default=False,
-                          help="Shows only ROS_PACKAGE_PATH separated by ':'. Supercedes all other options.",
-                          action="store_true")
-        parser.add_option("--only", dest="only", default=False,
-                          help="Shows comma-separated lists of only given comma-separated attribute(s).",
-                          action="store")
-        parser.add_option("--yaml", dest="yaml", default=False,
-                          help="Shows only version of single entry. Intended for scripting.",
-                          action="store_true")
-
+""" % {'prog': self.progname, 'opts': ONLY_OPTION_VALID_ATTRS},
+            epilog="See: http://www.ros.org/wiki/rosinstall for details\n")
+        parser.add_option(
+            "--data-only", dest="data_only", default=False,
+            help="Does not provide explanations",
+            action="store_true")
+        parser.add_option(
+            "--no-pkg-path", dest="no_pkg_path", default=False,
+            help="Suppress ROS_PACKAGE_PATH.",
+            action="store_true")
+        parser.add_option(
+            "--pkg-path-only", dest="pkg_path_only", default=False,
+            help="Shows only ROS_PACKAGE_PATH separated by ':'. Supercedes all other options.",
+            action="store_true")
+        parser.add_option(
+            "--only", dest="only", default=False,
+            help="Shows comma-separated lists of only given comma-separated attribute(s).",
+            action="store")
+        parser.add_option(
+            "--yaml", dest="yaml", default=False,
+            help="Shows only version of single entry. Intended for scripting.",
+            action="store_true")
+        parser.add_option(
+            "-u", "--untracked", dest="untracked",
+            default=False,
+            help="Also show untracked files as modifications",
+            action="store_true")
         # -t option required here for help but used one layer above, see cli_common
-        parser.add_option("-t", "--target-workspace", dest="workspace", default=None,
-                          help="which workspace to use",
-                          action="store")
+        parser.add_option(
+            "-t", "--target-workspace", dest="workspace", default=None,
+            help="which workspace to use",
+            action="store")
         (options, args) = parser.parse_args(argv)
 
         if config is None:
@@ -275,75 +295,56 @@ $ %(prog)s info --only=path,cur_uri,cur_revision robot_model geometry
                 additional_uris=[],
                 config_filename=self.config_filename)
         elif config.get_base_path() != target_path:
-            raise MultiProjectException("Config path does not match %s %s " % (config.get_base_path(), target_path))
+            raise MultiProjectException("Config path does not match %s %s " %
+                                        (config.get_base_path(), target_path))
+
         if args == []:
             args = None
-        # relevant for code completion, so these should yield quick response:
+
         if options.pkg_path_only:
             print(":".join(get_ros_package_path(config)))
             return 0
+        if options.no_pkg_path:
+            header = 'workspace: %s\nROS_ROOT: %s' % (target_path,
+                                                      get_ros_stack_path(config))
+            print(header)
+            return 0
         elif options.only:
             only_options = options.only.split(",")
             if only_options == '':
                 parser.error('No valid options given')
-
-            lookup_required = False
-            for attr in only_options:
-                if not attr in only_option_valid_attrs:
-                    parser.error("Invalid --only option '%s', valids are %s" % (attr, only_option_valid_attrs))
-                if attr in ['cur_revision', 'cur_uri', 'revision']:
-                    lookup_required = True
-            elements = select_elements(config, args)
-            for element in elements:
-                if lookup_required and element.is_vcs_element():
-                    spec = element.get_versioned_path_spec()
-                else:
-                    spec = element.get_path_spec()
-                output = []
-                for attr in only_options:
-                    if 'localname' == attr:
-                        output.append(spec.get_local_name() or '')
-                    if 'path' == attr:
-                        output.append(spec.get_path() or '')
-                    if 'scmtype' == attr:
-                        output.append(spec.get_scmtype() or '')
-                    if 'uri' == attr:
-                        output.append(spec.get_uri() or '')
-                    if 'version' == attr:
-                        output.append(spec.get_version() or '')
-                    if 'revision' == attr:
-                        output.append(spec.get_revision() or '')
-                    if 'cur_uri' == attr:
-                        output.append(spec.get_curr_uri() or '')
-                    if 'cur_revision' == attr:
-                        output.append(spec.get_current_revision() or '')
-                print(','.join(output))
+            lines = get_info_table_raw_csv(config,
+                                           properties=only_options,
+                                           localnames=args)
+            print('\n'.join(lines))
             return 0
-        if options.yaml:
+        elif options.yaml:
             source_aggregate = cmd_snapshot(config, localnames=args)
-            print(yaml.safe_dump(source_aggregate))
+            print(yaml.safe_dump(source_aggregate), end='')
             return 0
 
         # this call takes long, as it invokes scms.
-        outputs = cmd_info(config, localnames=args)
-        if args is not None and len(outputs) == 1:
+        outputs = cmd_info(config, localnames=args, untracked=options.untracked)
+        if args and len(args) == 1:
+            # if only one element selected, print just one line
             print(get_info_list(config.get_base_path(),
                                 outputs[0],
                                 options.data_only))
             return 0
 
         header = 'workspace: %s\nROS_ROOT: %s' % (target_path,
-                                                get_ros_stack_path(config))
+                                                  get_ros_stack_path(config))
         print(header)
-        if not options.no_pkg_path:
-            table = get_info_table(config.get_base_path(),
-                                   outputs,
-                                   options.data_only,
-                                   reverse=reverse)
-            if table is not None and table != '':
-                print("\n%s" % table)
+        table = get_info_table(config.get_base_path(),
+                               outputs,
+                               options.data_only,
+                               reverse=reverse)
+        if table is not None and table != '':
+            print("\n%s" % table)
 
         return 0
+
+
 def rosws_main(argv=None, usage=None):
     """
     Calls the function corresponding to the first argument.
@@ -356,7 +357,8 @@ def rosws_main(argv=None, usage=None):
     if (sys.argv[0] == '-c'):
         sys.argv = [_PROGNAME] + sys.argv[1:]
     if '--version' in argv:
-        print("%s: \t%s\n%s" % (_PROGNAME, rosinstall.__version__.version, cmd_version()))
+        print("%s: \t%s\n%s" %
+              (_PROGNAME, rosinstall.__version__.version, cmd_version()))
         sys.exit(0)
 
     if not usage:
@@ -378,7 +380,7 @@ def rosws_main(argv=None, usage=None):
             usage()
             return 0
 
-    if '--help' == argv[1]:
+    if argv[1] in ['--help', '-h']:
         usage()
         return 0
 
@@ -414,8 +416,9 @@ def rosws_main(argv=None, usage=None):
             'status': cli.cmd_status,
             'update': cli.cmd_update}
         for label in list(ws_commands.keys()):
-            if label in __MULTIPRO_CMD_ALIASES__:
-                ws_commands[__MULTIPRO_CMD_ALIASES__[label]] = ws_commands[label]
+            alias = __MULTIPRO_CMD_ALIASES__.get(label, None)
+            if alias:
+                ws_commands[alias] = ws_commands[label]
 
         if command not in commands and command not in ws_commands:
             if os.path.exists(command):
@@ -423,7 +426,8 @@ def rosws_main(argv=None, usage=None):
                 command = 'info'
             else:
                 if command.startswith('-'):
-                    print("First argument must be name of a command: %s" % command)
+                    print("First argument must be name of a command: %s" %
+                          command)
                 else:
                     print("Error: unknown command: %s" % command)
                 usage()
